@@ -18,7 +18,7 @@
 # the Free Software Foundation, Inc., 51 Franklin Street,
 # Boston, MA 02110-1301, USA.
 
-from labjack_u12 import LabjackU12
+from u12 import LabjackU12
 
 from enthought.traits.api import HasTraits
 from enthought.traits.api import Str, List as TList, Instance, Bool
